# -*- mode: ruby; coding: utf-8 -*-

base_dir = File.join(File.dirname(__FILE__),
                     "..", "..", "..", "..", "ruby", "groonga")
$LOAD_PATH.unshift(File.join(base_dir, "ext"))
$LOAD_PATH.unshift(File.join(base_dir, "lib"))

require 'rubygems'
require 'rack'
require 'groonga'

use Rack::CommonLogger
use Rack::Static, :urls => ["/css/", "/images/"], :root => "public"

Groonga::Database.create

class Keyword
  include Rack::Utils

  def initialize
    @text = File.read(File.join(File.dirname(__FILE__),
                                "ldd10-winter-program.txt"))
  end

  def call(env)
    request = Rack::Request.new(env)
    response = Rack::Response.new
    response["Content-Type"] = "text/html; charset=UTF-8"
    if /\/\z/ !~ request.path_info
      request.path_info += "/"
      response.redirect(request.url)
      return response.to_a
    end

    response.write(<<-EOH)
<?xml version="1.0" encoding="utf-8"?>
<!DOCTYPE html PUBLIC "-//W3C//DTD XHTML 1.0 Transitional//EN"
       "http://www.w3.org/TR/xhtml1/DTD/xhtml1-transitional.dtd">
<html xmlns="http://www.w3.org/1999/xhtml" xml:lang="ja" lang="ja">
<head>
  <meta http-equiv="content-type" content="text/html; charset=UTF-8" />
  <meta name="robot" content="noindex,nofollow" />
  <title>groongaでキーワード検出</title>
  <link rel="stylesheet" href="css/keyword.css" type="text/css" media="all" />
</head>
<body>
<div class="header">
  <h1>groongaでキーワード検出</h1>
</div>

<div class="content">
EOH

    render_keyword_box(request, response)
    render_keyword_detection_result(request, response)

    response.write(<<-EOF)
</div>

<div class="footer">
<p class="powered-by-groonga">
  <a href="http://groonga.org/">groonga</a>
  #{Groonga::VERSION.join('.')}を使っています。
</p>
<p class="powered-by-ruby-groonga">
  <a href="http://groonga.rubyforge.org/">Ruby/groonga</a>
  #{Groonga::BINDINGS_VERSION.join('.')}を使っています。
</p>
</div>

</body>
</html>
EOF
    response.to_a
  end

  private
  def keywords(request)
    (request['keywords'] || '').split
  end

  def render_keyword_box(request, response)
    response.write(<<-EOF)
<div class="keyword-box">
  <form method="get" action="./">
    <p>
      <input name="keywords" type="text"
             value="#{escape_html(keywords(request).join(' '))}" />
      <input type="submit" value="検出" />
    </p>
  </form>
</div>
EOF
  end

  def render_keyword_detection_result(request, response)
    words = Groonga::PatriciaTrie.create(:key_type => "ShortText",
                                         :key_normalize => true)
    keywords(request).each do |keyword|
      words.add(keyword)
    end
    text = words.tag_keys(escape_html(@text)) do |record, word|
      "<span class='keyword'>#{word}</span>"
    end
    response.write(<<-EORH)
  <div class='keyword-detect-result'>
#{text.gsub(/\n/, "<br />")}
  </div>
EORH
  end
end

run Keyword.new
